@@ -1,11 +1,10 @@
 #pragma once
 
-#include "THCGeneral.h"
+#include <THC/THCGeneral.h>
 
 /* Global state of THC. */
 struct THCState {
   struct THCRNGState* rngState;
-  struct cudaDeviceProp* deviceProperties;
   /* Set of all allocated resources. blasHandles and sparseHandles do not have
      a default and must be explicitly initialized. We always initialize 1
      blasHandle and 1 sparseHandle but we can use more.
